#include "TetrisFigure.h"

CTetrisFigure::CTetrisFigure()
{
	m_ppBlocks = NULL;
	m_iBlocksCount = 0;
}

CTetrisFigure::~CTetrisFigure()
{
	Destroy();
}

void CTetrisFigure::Create(int num_blocks)
{
	if(m_ppBlocks != NULL)
		Destroy();
	else
	{
		if(num_blocks > 0)
		{
			m_iBlocksCount = num_blocks;

			m_ppBlocks = new CTetrisBlock*[num_blocks];
			for (int i=0; i<num_blocks; i++)
			{
				m_ppBlocks[i] = new CTetrisBlock;
				m_ppBlocks[i]->SetPosX(gNumBlocksX / 2);
				m_ppBlocks[i]->SetPosY(gNumBlocksY - 2);
			}
		}
	}
}

/****************** NEW !!!!!! **************************************************************************/
/*
	Here we are loading our tetris figure blocks by going thru each child of a given
	XML root element.
*/
int CTetrisFigure::LoadFromFile(char *filename, TiXmlNode *this_node)
{
	TiXmlNode *main_node = NULL;

	TiXmlDocument doc(filename);
	if(filename !=NULL)
	{
		bool loadOkay = doc.LoadFile();

		if ( !loadOkay )
			return false;

		main_node = doc.FirstChild();
	}
	else
		main_node = this_node;

	// Remember, a figure is simply an array of blocks, so it might not have any additional properties
	const char *value = NULL;
	TiXmlElement* element = NULL;
	element = main_node->ToElement();

	TiXmlNode *child_node = NULL;
	child_node = main_node->FirstChild();

	int num_blocks = 0;

	while(child_node !=NULL)
	{
		element = child_node->ToElement();

		if((stricmp(child_node->Value(), "block") == 0))
			num_blocks++;

		child_node = main_node->IterateChildren(child_node);
	}

	Destroy();
	Create(num_blocks);

	int count = 0;
	while(child_node !=NULL)
	{
		element = child_node->ToElement();

		if((stricmp(child_node->Value(), "block") == 0) && (stricmp(element->Attribute("Type"), "template") == 0))
		{
			if(!m_ppBlocks[count]->LoadFromFile(NULL, child_node))
				return -1;
		}

		child_node = main_node->IterateChildren(child_node);
	}

	return 1;
}

void CTetrisFigure::Destroy()
{
	if(m_ppBlocks)
	{
		for (int i=0; i<GetBlockCount(); i++)
		{
			if(m_ppBlocks[i] !=NULL)
				delete m_ppBlocks[i];
		}
		delete[] m_ppBlocks;
	}

	m_ppBlocks = NULL;
}

void CTetrisFigure::Draw()
{
	if(!m_ppBlocks)
		return;

	for (int i=0; i < GetBlockCount(); i++)
	{
		if(m_ppBlocks[i] !=NULL)
			m_ppBlocks[i]->Draw();
	}
}

/***************************** NEW !!!!! ************************************************/
/*	Here we do rotation and rotation tests for each of the block. If each and every block
	can be rotated, return true, otherwise return false.
*/
bool CTetrisFigure::Rotate(int dir, bool can_rotate_test)
{
	if(can_rotate_test)
	{
		for (int i=0; i<GetBlockCount(); i++)
		{
			if(m_ppBlocks[i] !=NULL)
				if(!m_ppBlocks[i]->Rotate(dir, m_ppBlocks[0]->GetPosX(), m_ppBlocks[0]->GetPosY(), true))
					return false;
		}
	}
	else
	{
		for (int i=0; i<GetBlockCount(); i++)
		{
			if(m_ppBlocks[i] !=NULL)
				m_ppBlocks[i]->Rotate(dir, m_ppBlocks[0]->GetPosX(), m_ppBlocks[0]->GetPosY(), false);
		}
	}

	
	return true;
}
/***************************** NEW !!!!! ************************************************/

/***************************** NEW !!!!! ************************************************/
/*	Here we do movement and movement tests for each of the block. If each and every block
	can be moved, return true, otherwise return false.
*/
bool CTetrisFigure::Move(int dir, bool can_move_test)
{
	// We do testing
	if(can_move_test)
	{
		for (int i=0; i<GetBlockCount(); i++)
		{
			if(m_ppBlocks[i] !=NULL)
			{
				if(!m_ppBlocks[i]->Move(dir, true))
					return false;
			}
		}
	}
	else
	{
		for (int i=0; i<GetBlockCount(); i++)
		{
			if(m_ppBlocks[i] !=NULL)
				m_ppBlocks[i]->Move(dir, false);
		}
	}

	return true;
}
/***************************** NEW !!!!! ************************************************/

CTetrisBlock *CTetrisFigure::GetBlock(int x, int y)
{
	for (int i=0; i<GetBlockCount(); i++)
	{
		if(m_ppBlocks[i] !=NULL)
		{
			if(m_ppBlocks[i]->GetPosX() == x && m_ppBlocks[i]->GetPosY() == y)
				return m_ppBlocks[i];
		}
	}

	return NULL;
}

CTetrisBlock *CTetrisFigure::GetBlock(int index)
{
	if(index >=0 && index < GetBlockCount())
		return m_ppBlocks[index];

	return NULL;
}

int CTetrisFigure::GetBlockCount()
{
	return m_iBlocksCount;
}


