#include "TetrisGame.h"

CTetrisGame::CTetrisGame()
{
	m_pCurrentFigure = NULL;
	m_pNextFigure = NULL;
	m_ppTetrisMatrix = NULL;

	m_eGameState = TetrisGame_Normal;
	m_iScore = 0;
	m_iFullLine = -1;
	m_fLastTime = 0;
}

CTetrisGame::~CTetrisGame()
{
	Destroy();
}


void CTetrisGame::Destroy()
{
	if(m_pCurrentFigure !=NULL)
	{
		delete m_pCurrentFigure;
		m_pCurrentFigure = NULL;
	}

	if(m_pTemplateBlocks)
	{
		delete[] m_pTemplateBlocks;
		m_pTemplateBlocks = NULL;
	}

	if(m_pTemplateFigures)
	{
		delete[] m_pTemplateFigures;
		m_pTemplateFigures = NULL;
	}


	if(m_ppTetrisMatrix)
	{
		for (int y=0; y<gNumBlocksY; y++)
		{
			delete[] m_ppTetrisMatrix[y];
			m_ppTetrisMatrix[y] = NULL;
		}

		delete[] m_ppTetrisMatrix;
		m_ppTetrisMatrix = NULL;

	}

	m_pOldFigures.erase();

	m_iScore = 0;
}
/****************** NEW !!!!!! **************************************************************************/
/*
	Here we are loading our tetris game from XML file.
	Firstly, we count number of template figures (and blocks), and then load them.
	Also we load this game' belonged attributes, such as block size/spacing.
*/
int CTetrisGame::LoadFromFile(char *filename, TiXmlNode *this_node)
{
	TiXmlNode *main_node = NULL;

	TiXmlDocument doc(filename);
	if(filename !=NULL)
	{
		bool loadOkay = doc.LoadFile();

		if ( !loadOkay )
			return false;

		main_node = doc.FirstChild();
	}
	else
		main_node = this_node;

	// Remember, a figure is simply an array of blocks, so it might not have any additional properties
	const char *value = NULL;
	TiXmlElement* element = NULL;
	element = main_node->ToElement();

	value = element->Attribute("BlockSize");
	if(value !=NULL)
	{
		m_iBlockSize = atoi(value);
		value = NULL;
	}
	else 
		return 0;

	value = element->Attribute("BlockSpacing");
	if(value !=NULL)
	{
		m_iBlockSpacing = atoi(value);
		value = NULL;
	}
	else
		return 0;

	value = element->Attribute("Window");
	if(value !=NULL)
	{
		sscanf(value, "%d,%d,%d,%d", &m_GameWindow.m_iLeft, &m_GameWindow.m_iRight, &m_GameWindow.m_iTop, &m_GameWindow.m_iBottom);
		value = NULL;
	}
	else
		return -1;

		// Set global variables for this game
	gBlockSize = m_iBlockSize;
	gBlockSpacing = m_iBlockSpacing;

	gGameWindow = m_GameWindow;
	
	gNumBlocksX = gGameWindow.Width() / (gBlockSize + gBlockSpacing);
	gNumBlocksY = gGameWindow.Height() / (gBlockSize + gBlockSpacing);

	// OK, variables are set
	Destroy();
	Create();


	// Count the number of template figures/blocks
	TiXmlNode *child_node = NULL;
	child_node = main_node->FirstChild();

	while(child_node !=NULL)
	{
		element = child_node->ToElement();

		if((stricmp(child_node->Value(), "figure") == 0) && (stricmp(element->Attribute("Type"), "template") == 0))
			m_iNumTemplateFigures++;
		else if(stricmp(child_node->Value(), "block") == 0)
			m_iNumTemplateBlocks++;

		child_node = main_node->IterateChildren(child_node);
	}

	/*
	// Allocate space for template blocks/figures
	if(m_iNumTemplateBlocks > 0)
		m_pTemplateBlocks = new CTetrisBlock[m_iNumTemplateBlocks];
	else
		return 0;
	*/

	if(m_iNumTemplateFigures > 0)
		m_pTemplateFigures = new CTetrisFigure[m_iNumTemplateFigures];
	else
		return 0;

	child_node = main_node->FirstChild();
	int figure_count = 0, block_count = 0;

	while(child_node !=NULL)
	{
		if((stricmp(child_node->Value(), "figure") == 0) && (stricmp(element->Attribute("Type"), "template") == 0))
		{
			if(m_pTemplateFigures[figure_count].LoadFromFile(NULL, child_node))
				figure_count++;
			else
				return 0;
		}
		else if(stricmp(child_node->Value(), "block") == 0)
		{
			if(m_pTemplateBlocks[block_count].LoadFromFile(NULL, child_node))
				figure_count++;
			else
				return 0;	
		}

		child_node = main_node->IterateChildren(child_node);
	}

	return 1;
}

/************************ NEW !!!!! *********************************************/
/*
	Our Update() function. Here we do all checks vs game state, eliminate full 
	lines, and do some other stuff.
*/
void CTetrisGame::Update()
{
	if(m_eGameState == TetrisGame_Paused)
		return;

	if(m_eGameState == TetrisGame_Normal)
	{
		double CurrentTime = m_Timer.getAbsoluteTime();
		if(CurrentTime - m_fLastTime >= TIME_AQUIRE_USER_INPUT) // If the time given to aquire user input has passed
		{
			if(m_pCurrentFigure !=NULL && (Move(m_pCurrentFigure, DISPLACE_DOWN, true) == false)) // Our figure cannot move down anymore, map it to game blocks array
			{
				for (int i=0; i<gNumBlocksY; i++)
				{
					if(LineIsFull(i))
					{
						m_iFullLine = i;
						m_eGameState = TetrisGame_EliminatingLine;
						break;
					}
				}

				if(m_iFullLine == -1)
					DropRandomFigure();
			}
			else
				Move(m_pCurrentFigure, DISPLACE_DOWN, false);	// Our figure CAN be moved down, so move it now

			m_fLastTime = CurrentTime;
		}
	}

	if(m_eGameState == TetrisGame_EliminatingLine)
	{
		if(!AnimateLine(m_iFullLine)) // If AnimateLine returns false, it means that all blocks have finished their current animation
		{
			// Line blocks animations are complete, remove this line from game blocks array
			for (int i=0; i<gNumBlocksY; i++)
				m_ppTetrisMatrix[m_iFullLine][i] = NULL;

			// Shift above lines down 
			ShiftLines(m_iFullLine);
			
			// Restore memeber variables for next check
			m_eGameState = TetrisGame_Normal;
			m_iFullLine = -1;
		}
	}
}

/************************ NEW !!!!! *********************************************/

void CTetrisGame::Draw()
{
	Update();
	
	// Set global variables for this game
	gBlockSize = m_iBlockSize;
	gBlockSpacing = m_iBlockSpacing;

	gGameWindow = m_GameWindow;
	
	gNumBlocksX = gGameWindow.Width() / (gBlockSize + gBlockSpacing);
	gNumBlocksY = gGameWindow.Height() / (gBlockSize + gBlockSpacing);
	// OK, variables are set

	for (int i=0; i<gNumBlocksY; i++)
	{
		for (int j=0; j<gNumBlocksX; j++)
		{
			if(m_ppTetrisMatrix[i][j] !=NULL)
				m_ppTetrisMatrix[i][j]->Draw();
		}
	}
}

void CTetrisGame::Create()
{
	m_ppTetrisMatrix = new CTetrisBlock**[gNumBlocksY];

	for (int i=0; i<gNumBlocksY; i++)
	{
		m_ppTetrisMatrix[i] = new CTetrisBlock*[gNumBlocksX];
		for (int j=0; j<gNumBlocksX; j++)
			m_ppTetrisMatrix[i][j] = NULL;
	}
}

bool CTetrisGame::Move(CTetrisFigure *figure, int dir, bool can_move_test)
{
	if(figure == NULL)
		return false;

	if(can_move_test)
	{
		if(!figure ->Move(dir, true))
			return false;
		else			
		{
			CTetrisFigure tmp;
			tmp.Create(figure->GetBlockCount());

			for (int i=0; i<figure->GetBlockCount(); i++)
			{
				tmp.GetBlock(i)->SetPosX(figure->GetBlock(i)->GetPosX());
				tmp.GetBlock(i)->SetPosY(figure->GetBlock(i)->GetPosY());
			}

			tmp.Move(dir, false);
			
			bool all_good = true;
			for (i=0; i<tmp.GetBlockCount(); i++)
			{
				int block_at_x = tmp.GetBlock(i)->GetPosX();
				int block_at_y = tmp.GetBlock(i)->GetPosY();
				
				if(figure ->GetBlock(block_at_x, block_at_y) == NULL)
				{
					if(m_ppTetrisMatrix[block_at_y][block_at_x] != NULL)
						return false;
				}
				
			}
		}
	}
	else
	{
		MapFigure (figure , 0);
		figure ->Move(dir, false);
		MapFigure(figure , 1);
	}

	return true;
}

bool CTetrisGame::Rotate(CTetrisFigure *figure, int dir, bool can_rotate_test)
{
	if(figure == NULL)
		return false;

	if(can_rotate_test)
	{
		if(!figure->Rotate(dir, true)) 
			return false;

		CTetrisFigure tmp;
		tmp.Create(figure->GetBlockCount());

		for (int i=0; i<figure->GetBlockCount(); i++)
		{
			tmp.GetBlock(i)->SetPosX(figure->GetBlock(i)->GetPosX());
			tmp.GetBlock(i)->SetPosY(figure->GetBlock(i)->GetPosY());
		}

		tmp.Rotate(dir, false);
 
		for (i=0; i<tmp.GetBlockCount(); i++)
		{
			int block_at_x = tmp. GetBlock(i)->GetPosX();
			int block_at_y = tmp. GetBlock(i)->GetPosY();
					
			if(figure->GetBlock(block_at_x, block_at_y) == NULL)
			{
				if(m_ppTetrisMatrix[block_at_y][block_at_x] != NULL)
						return false;
			}
		}
	}
	else
	{
		MapFigure(figure, 0);
		figure->Rotate(dir, false);
		MapFigure (figure, 1);
	}

	return true;
}

void CTetrisGame::MapFigure(CTetrisFigure *figure, bool assign)
{
	if(figure == NULL)
		return;

	for (int i=0; i<figure->GetBlockCount(); i++)
	{
		if(assign)
		{
			if(figure->GetBlock(i) !=NULL)	
				m_ppTetrisMatrix[figure->GetBlock(i)->GetPosY()][figure->GetBlock(i)->GetPosX()] = figure->GetBlock(i);
		}
		else
		{
			if(figure->GetBlock(i) !=NULL)	
				m_ppTetrisMatrix[figure->GetBlock(i)->GetPosY()][figure->GetBlock(i)->GetPosX()] = NULL;
		}
	}
}

void CTetrisGame::DropRandomFigure()
{
	if(m_pCurrentFigure !=NULL)
		m_pOldFigures.push_back(m_pCurrentFigure);

	CTetrisFigure *figure = new CTetrisFigure;
	figure->Create(4);

	figure->GetBlock(0)->SetPosX(gNumBlocksX / 2);
	figure->GetBlock(0)->SetPosY(gNumBlocksY - 2);
	
	figure->GetBlock(1)->SetPosX((gNumBlocksX / 2) + 1);
	figure->GetBlock(1)->SetPosY(gNumBlocksY - 2);

	figure->GetBlock(2)->SetPosX((gNumBlocksX / 2) + 1);
	figure->GetBlock(2)->SetPosY(gNumBlocksY - 1);

	figure->GetBlock(3)->SetPosX(gNumBlocksX / 2);
	figure->GetBlock(3)->SetPosY(gNumBlocksY - 1);

	m_pCurrentFigure = figure;
	MapFigure(m_pCurrentFigure, true);
}

bool CTetrisGame::LineIsFull(unsigned int line_index)
{
	for (int x=0; x<gNumBlocksX; x++)
	{
		if(m_ppTetrisMatrix[line_index][x] == NULL)
			return false;
	}

	return true;
}

void CTetrisGame::ShiftLines(unsigned int line_eliminated)
{
	for (int i=line_eliminated+1; i<gNumBlocksY; i++) // Remeber, we are shifting lines that are above the eliminated one
	{
		// Shift this line down
		for (int j=0; j<gNumBlocksX; j++)
		{
			// Move line down with coordinates
			if(m_ppTetrisMatrix[i][j] !=NULL)
				m_ppTetrisMatrix[i][j]->Move(DISPLACE_DOWN, false);

			// Move line down in pointers
			m_ppTetrisMatrix[i-1][j] = m_ppTetrisMatrix[i][j];
			// Eliminate moved line in pointers
			m_ppTetrisMatrix[i][j] = NULL;
		}
	}		
}

bool CTetrisGame::AnimateLine(unsigned int line_index)
{

	return false;
}

CTetrisFigure *CTetrisGame::GetFigure()
{
	return m_pCurrentFigure;
}
