#ifndef TETRIS_GAME_INCLUDED
#define TETRIS_GAME_INCLUDED

#include "TetrisFigure.h"

enum eTetrisGameState
{
	TetrisGame_Normal,
	TetrisGame_Paused,
	TetrisGame_EliminatingLine
};

class CTetrisGame
{
public:
		
	void Create();
	void Draw();
	void Destroy();
		
	void Update();

	/*
		Those functions were updated to handle a specified figure, not only 
		the current one in control.
	*/
	bool Rotate(CTetrisFigure *figure, int dir, bool can_rotate_test);
	bool Move(CTetrisFigure *figure, int dir, bool can_move_test);
	

	CTetrisFigure *GetFigure();

	void DropRandomFigure();
	void MapFigure(CTetrisFigure *figure, bool assign);

	/* This function will handle the animation of the given line */
	bool AnimateLine(unsigned int line_index);

	/* This function will shift all lines from above downwards*/
	void ShiftLines(unsigned int line_eliminated);
	
	/* This function returns true if the given line is full, false otherwise*/
	bool LineIsFull(unsigned int line_index);
	
	/****************** NEW !!!!! ***********************************************/
	/*	This function loads an XML file and parses it's data */
	virtual int LoadFromFile(char *filename, TiXmlNode *node);
	/****************** NEW !!!!! ***********************************************/

	CTetrisGame();
	virtual ~CTetrisGame();
	
private:

	/****************** NEW !!!!! ***********************************************/
	/*	This variable contains the tetris game window coordinates 
		(remember, there might be more than one game on screen running)
	*/
	tRect m_GameWindow;

	// Block spacing and block size variables (no more 'define's, use variables)
	unsigned int m_iBlockSpacing;
	unsigned int m_iBlockSize;

	/****************** NEW !!!!! ***********************************************/

	/* This variables are going to watch after:	*/
	eTetrisGameState m_eGameState;	// Current game state - paused, normal ,etc
	int m_iFullLine;				// A full line index

	CTimer m_Timer;
	double m_fLastTime;

	int m_iScore;
	
	CTetrisFigure *m_pTemplateFigures;
	CTetrisBlock *m_pTemplateBlocks;

	int m_iNumTemplateFigures;
	int m_iNumTemplateBlocks;

	CTetrisBlock ***m_ppTetrisMatrix;
	
	CTetrisFigure *m_pCurrentFigure;
	CTetrisFigure *m_pNextFigure;

	CDoubleList<CTetrisFigure *> m_pOldFigures;
};

#endif